#!/bin/sh
#

ACS_INSTALL_DIR="%%INSTALL_DIR%%"
SCRIPT_NAME=`basename $0`;

usage()
{
    echo ""
    echo "usage: ${SCRIPT_NAME}  [ -a <action> ] ";
    echo "                -d <DB2_Instance_Directory>  ";
    echo "                -u <Instance_user_ID_name> ";
    echo "                -g <Instance_primary_group_name> ";
    echo "                 [ -F ] forces ${SCRIPT_NAME} to install in spite of a newer already installed version";
    echo "";
}

checkbin()
{
    if test -f ${1} -a -x ${1}; then
        if test "${CHECK_VERBOSITY}" = "verbose"; then
            echo ""
            echo "checking ${1} ... "
            ${1} -f test 2>&1
        else
            ${1} -f test > /dev/null 2>&1
        fi
    
        if test $? -eq 0; then
            if test "${CHECK_VERBOSITY}" = "verbose"; then
                echo "OK"
            fi
            ACS_AGENT=${1};
            return 1;
        fi
    else
        if test "${CHECK_VERBOSITY}" = "verbose"; then
            echo ""
            echo "File ${1} not found."
        fi
    fi
    return 0;
}

enableSUID()
{
    SUB_RC=2;
    if test -f ${1} -a -x ${1}; then
	SUB_RC=0;
	chown root ${1}
	let SUB_RC=${SUB_RC}+$?;
	chmod 4750 ${1}
	let SUB_RC=${SUB_RC}+$?;
    fi
    return $SUB_RC;
}

yes_or_no()
{
    while true ; do
        echo "$*";
        read response;
        case $response in
            "yes"|"Yes"|"YES"|"y"|"Y") return 0;;
            "no"|"No"|"NO"|"n"|"N") return 1;;
            *) echo "    Please answer either yes or no.";
        esac;
    done;
}

copyFiles()
{
  EXEC=`ls | grep -E '%%FILES%%' | xargs -i cp -r {} ${INST_DIR}/acs/`
  SUB_RC=$?	
  return $SUB_RC;
}

read_path()
{
    while true ; do
	echo "Please enter the directory where the profile will be stored: [${PROFILE_PATH}]"
	read INPUT;
	if ! test -z "${INPUT}"; then
	    PROFILE_PATH=$INPUT;
	fi
	if test -d ${PROFILE_PATH}; then
	    return 0;
	else
	    yes_or_no "The directory ${PROFILE_PATH} does not exist. Do you want me to create it? [Y/N]";
	    if test $? -eq 0; then
		mkdir ${PROFILE_PATH};
		if test $? -eq 0; then
		    return 0;
		fi
	    fi
	fi
    done
}

read_link()
{
    LINKED_FILE=$1;
    while test -h "${LINKED_FILE}"; do
	TRGT=`ls -l ${LINKED_FILE} | sed 's/^.*-> //'`;
	FIRST=`echo ${TRGT} | cut -c 1,1`;
	if test ${FIRST} = "/"; then
	    LINKED_FILE=${TRGT};
	else
	    DIR=`dirname ${LINKED_FILE}`;
	    LINKED_FILE=${DIR}/${TRGT};
	fi
    done
}

select_profile_action()
{
    while test true; do
	echo "Found profile "${PROFILE};
	echo "Do you want to:";
	echo " (c)reate a new profile";
	echo " (r)euse the profile unchanged";
	echo " (m)odify the profile";
	echo "Please enter one of the strings in parentheses:"
	read SEL;
	case $SEL in
	    "c")
		echo "Please remove or move away $PROFILE and restart ${SCRIPT_NAME}.";
		exit 2;
	    ;;
	    "r")
		WIZARD=0;
		return 0;
	    ;;
	    "m")
		WIZARD=1;
		return 0;
	    ;;
	    *)
		echo "Invalid input: ${SEL}";
	    ;;
	esac	
    done
}

install_offload_agent()
{
    echo "";
    echo "The backup to TSM will be performed by the executable 'tsm4acs'.";
    echo "If you want to delay the backup until necessary resources are available";
    echo "you can start it manually or by using a scheduler.";
    echo "If you want to perform the backup to TSM immediately after the flashcopy is";
    echo "finished you can request now that 'tsm4acs' is added to the /etc/inittab.";
    echo "";
    yes_or_no "Do you want to add 'tsm4acs' to the /etc/inittab? [Y/N]";
    if test $? -eq 0; then
	OFFLOAD_ARG="offload";
    else
	OFFLOAD_ARG="nooffload";
    fi
    while true ; do
	echo "On which type of storage device do the database files reside on?"
	if test ${ACSCIM} -eq 1; then
	    echo " DS8000/DS6000/SVC (CIM)"
	fi
	if test ${ACSNSAN} -eq 1; then
	    echo " N-SERIES SAN (NSAN)"
	fi
	if test ${ACSNNAS} -eq 1; then
	    echo " N-SERIES NAS (NNAS)"
	fi
    if test ${ACSXIV} -eq 1; then
	    echo " IBM XIV Nextra (XIV)"
	fi
	echo "Please enter one of the strings in parentheses:"
	read AGENT_TYPE;
	AGENT_TYPE=`echo ${AGENT_TYPE} | tr [a-z] [A-Z]`;
	case $AGENT_TYPE in
	    "CIM")
	    if test ${ACSCIM} -eq 1; then
		${INST_DIR}/acs/acscim -f install_BS_${OFFLOAD_ARG}  -p ${PROFILE}
		return $?;
	    fi
	    ;;
	    "NSAN")
	    if test ${ACSNSAN} -eq 1; then
		${INST_DIR}/acs/acsnsan -f install_BS_${OFFLOAD_ARG} -p ${PROFILE}
		return $?;
	    fi
	    ;;
	    "NNAS")
	    if test ${ACSNNAS} -eq 1; then
		${INST_DIR}/acs/acsnnas -f install_BS_${OFFLOAD_ARG} -p ${PROFILE}
		return $?;
	    fi
	    ;;
        "XIV")
	    if test ${ACSXIV} -eq 1; then
		${INST_DIR}/acs/acsxiv -f install_BS_${OFFLOAD_ARG} -p ${PROFILE}
		return $?;
	    fi
	    ;;
	    *)
	    echo "Your input '$AGENT_TYPE' is not valid."
	esac
    done
}

check_perm()
{
    # check permissions of the agent
    # we can not check the owner because this might be only a link to the real executable
    if test -u ${1}; then
	if test "${2}" = "verbose"; then
	    echo "SUID bit for ${1} is set. Ok.";
	fi
	return 0;
    else
	echo "SUID bit for ${1} is NOT set. Please run '${SCRIPT_NAME} -a enable -d <DB2_Instance_Directory>' as root.";
    fi
    return 1;
}

# default if no action is specified via cmd line
ACTION=setup
CHECK_VERBOSITY=verbose

# parse the commandline options
FORCE_INSTALL=0;
while test ${#} -gt 0; do
    case ${1} in
        "-a")
	    case ${2} in
	    	start);;
		query);;
		list);;
		install);;
		enable);;
		disable)CHECK_VERBOSITY="";;
		stop)CHECK_VERBOSITY="";;
		*)
		    echo "Action ${2} is not supported";
		    usage;
		    exit 2;
		;;
	    esac
	    ACTION=${2}
            shift
	;;
	-d)
	    INST_DIR=${2};
            shift
	;;
	-u)
	    INST_USER=${2};
            shift
	;;
	-g)
	    INST_GRP=${2};
            shift
	;;
	-F)
	    FORCE_INSTALL=1;
	;;
	*)
	    echo "unrecognized options ${1} ${2}";
	    usage;
	    exit 2;
	;;
    esac
    shift
done

# check if we have processed all options
if test ${#} -gt 0; then
    echo "unrecognized option ${1}";
    usage;
    exit 2;
fi

# instance directory is mandatory for all actions except setup and list
if test ${ACTION} != "setup"; then
    if test ${ACTION} != "list"; then
         if test -z "${INST_DIR}"; then
	     echo "Instance directory not specified.";
	     usage;
	     exit 2;
	 fi
    fi
else
    cd ${0%/*}
    CURDIR=`pwd`;
    cd ..
    INST_DIR=`pwd`;
    if test ! -d ${INST_DIR}/acs; then # not called from <instanceDir>/acs -> ask for instance dir and start from there or ask for user and group too and install
        DONE=0;
        
        echo "Script not started from 'acs' sub-directory of the DB2 instance. Trying to locate the correct instance or install a new instance...";
        
        while test $DONE -eq 0; do
            echo "  Please enter the instance directory: ";
            read instDir;
            if test -d $instDir; then
                DONE=1;
            else
                echo "    Directory does not exist, specify a vaild directory!";
            fi
        done;
        
        if test -f $instDir/acs/$SCRIPT_NAME; then
            echo "Instance located, starting the setup script from there.";
            if test `id -u` -ne 0; then # not root
        	    $instDir/acs/$SCRIPT_NAME;
            else # root
                echo "Instance activation is not possible as root user.";
                echo "  Please enter the user name of the db instance user: ";
                read instUser;
                CMD="su - $instUser '$instDir/acs/$SCRIPT_NAME'";
                $CMD;
        	fi
        else
            if test `id -u` -ne 0; then # not root
                echo "Unable to locate the specified instance. The installation/enabling/activation of a new instance is only possible as root.";
                usage;
                exit 2;
            else # root
                echo "This is a new instance, so we are going to install ACS there.";
                echo "  Please enter the name of the db instance user: ";
                read instUser;
                echo "  Please enter the group name of the db instance user: ";
                read instGroup;        
                cd $CURDIR && ./$SCRIPT_NAME -a install -d $instDir -u $instUser -g $instGroup;
                cd $instDir/acs && ./$SCRIPT_NAME -a enable -d $instDir -u $instUser -g $instGroup;
                CMD="su - $instUser '$instDir/acs/$SCRIPT_NAME'";
                $CMD;
            fi
        fi
        
        exit 0;
    	#echo "When running the setup wizard, this script must be located in the 'acs' subdir of the DB2 instance. Aborting.";
    	#exit 2;
    fi
fi

if test ${ACTION} != "list"; then
   if test ! -d ${INST_DIR}; then
      echo "";
      echo "";
      echo "Directory '$INST_DIR' not found. Please ensure that the directory exists ('INSTANCE HOME/sqllib'). Aborting.";
      echo "";
      echo "";
      exit 2;
   fi
   if test ! -d ${INST_DIR}/acs; then  
      echo "";
      echo "";
      echo "DB2 ACS installation directory '${INST_DIR}/acs' not found. Please make sure you have installed DB2 ACS with a valid license. Aborting.";
      echo "";
      echo "";
      exit 2;
   fi 

   # find an agent that works
   ACS_AGENT=
   checkbin ${INST_DIR}/acs/acsnnas
   ACSNNAS=$?;
   if test `uname` = "AIX"; then
   # those do not exist on Linux
       checkbin ${INST_DIR}/acs/acsnsan
       ACSNSAN=$?;
       checkbin ${INST_DIR}/acs/acscim
       ACSCIM=$?;
       checkbin ${INST_DIR}/acs/acsxiv
       ACSXIV=$?;
   else
       ACSNSAN=0;
       ACSCIM=0;
   fi
   
   if test "${ACS_AGENT}" = ""; then
       echo ""
       echo "No valid acs agent found. Aborting.";
       exit 2;
   fi
fi

echo "";
RC=2;

case ${ACTION} in
    start)
	if check_perm ${ACS_AGENT}; then
	    ${ACS_AGENT} -f restart
	    RC=$?
	else
	    RC=2;
	fi
	;;
    stop)
	if check_perm ${ACS_AGENT}; then
	    ${ACS_AGENT} -f stop
	    RC=$?
	else
	    RC=2;
	fi
	;;
    enable)
	if test `id -u` -ne 0; then
	    echo "You must be root to perform this action. Aborting";
	    exit 2;
	fi
# ensure proper permissions for all agents
	NUM_VALID_AGENTS=0
	enableSUID ${INST_DIR}/acs/acscim
	if test $? -eq 0; then let NUM_VALID_AGENTS=$NUM_VALID_AGENTS+1; fi
	enableSUID ${INST_DIR}/acs/acsnsan
	if test $? -eq 0; then let NUM_VALID_AGENTS=$NUM_VALID_AGENTS+1; fi
	enableSUID ${INST_DIR}/acs/acsnnas
	if test $? -eq 0; then let NUM_VALID_AGENTS=$NUM_VALID_AGENTS+1; fi
    enableSUID ${INST_DIR}/acs/acsxiv
	if test $? -eq 0; then let NUM_VALID_AGENTS=$NUM_VALID_AGENTS+1; fi
	if test ${NUM_VALID_AGENTS} -eq 0; then
	    echo "Failed to set the SUID bit for any of the agents. Aborting.";
	    exit 2;
	else
	    RC=0;
	fi
	;;
    disable)
# stop the daemon and remove inittab entries
	if check_perm ${ACS_AGENT}; then
	    ${ACS_AGENT} -f stop
	    RC=$?
	else
	    RC=2;
	fi
	if test ${RC} -ne 0; then
	    echo "Unable to stop ACSD. Aborting";
	else
	    ${ACS_AGENT} -f uninstall
	    RC=$?
	fi
	;;
    list)
# list DB2 instances, where product could be installed/updated to
         if [ -f /usr/local/bin/db2ls ]; then
           DB2VERSION9_5=$(/usr/local/bin/db2ls -c|tail +2|awk -F':' '{ if ( match($2,"[9].[5-9].*") != 0 ) print $1}')
         else
           echo "";
           echo "";
           echo "Couldn't list DB2 UDB instances, db2ls in /usr/local/bin doesn't exist. Aborting.";
           echo "";
           echo "";
           exit 2;
         fi

         if [ -n "${DB2VERSION9_5}" ]; then
           echo "";
           echo "";
           echo "Instances of DB2 UDB V9.5 (or higher) found on this machine"
           echo "";
           echo "";
           for V9PATH in $(echo ${DB2VERSION9_5})
              do
                echo "found in ${V9PATH}:";
                echo "";
                INSTANCES9_5=$(${V9PATH}/instance/db2ilist)
                INSTANCES9_5RC=$?
                if [ ${INSTANCES9_5RC} = 0 ]; then
                  echo "${INSTANCES9_5}";
                fi
                echo "";
           done
         fi
         RC=$INSTANCES9_5RC
        ;;
    install)


# install/upgrade to/for specified instance
	
	# must be executed as root
    if test `id -u` -ne 0; then
        echo "You must be root to perform this action. Aborting.";
        exit 2;
    fi
	echo "";
    echo "Installing/Updating DB2 ACS installation for ${INST_DIR}/acs";
    echo "";
	echo "Checking environment";
    echo "- - - - - - - - - - -";
        
	# directory must exist
	if ! test -d ${INST_DIR}; then
        echo "- FAILED: installation directory doesn't exist.";
    	let RC=2
	else
	    echo "+ OK: installation directory exists.";
    	let RC=0
	fi

	#setup.sh must be executed in /opt/tivoli/tsm/ACS_VERSIONNUMBER
	TEMP_DIR=`pwd`;
	if [ $TEMP_DIR != "$ACS_INSTALL_DIR" ]; then
        echo "- FAILED: Please change to '${ACS_INSTALL_DIR}' and run '${SCRIPT_NAME}' again.";
        let RC=$RC+2
	else
	    echo "+ OK: ${SCRIPT_NAME} runs in ACS installation directory '${ACS_INSTALL_DIR}.'";
        let RC=$RC+0
	fi
    
    #check if installation is an update or a new installation
	UPDATE=0
	if test -u ${ACS_AGENT}; then
	    echo "+ OK: DB2 ACS installation exists in instance and will be upgraded."	
	    UPDATE=1
	else
        echo "+ OK: DB2 ACS installation will be installed to instance for the first time.";
	fi
	
	#check version
	VERSION_OLD=`${INST_DIR}/acs/acsd -v | sed '/Build/!d' | sed 's/.*Build //;s/).*//'`;
	VERSION_NEW=`${ACS_INSTALL_DIR}/acsd -v | sed '/Build/!d' | sed 's/.*Build //;s/).*//'`;
	COMPARE=`echo "$VERSION_NEW:$VERSION_OLD" | awk -F ':' '{ if ( $1 >= $2 ) { print "ok" } else { print "error" } }'`;
    if [ $COMPARE = "error" ]; then
	    echo "- FAILED: Version in instance home 'Build $VERSION_OLD' is newer than version in installation folder: 'Build $VERSION_NEW.";
	    if [ $FORCE_INSTALL -eq 1 ]; then
	       echo "+ OK: force install selected !";
           let RC=$RC+0
	    else
           echo "          Use option -F to force installation.";
           let RC=$RC+2
	    fi
	else
	    echo "+ OK: Version in instance home 'Build $VERSION_OLD' is older or equal than version in installation folder: 'Build $VERSION_NEW'.";
        let RC=$RC+0
	fi
	
	echo "";
    echo "Installation messages";
	echo "- - - - - - - - - - -";
	#if all of the checks before returned OK, we can install
	if test $RC -eq 0; then
        #new installation without s-bit set
        if test $UPDATE -eq 0; then
            echo "+ OK: New installation !";
            copyFiles
            if test $? -eq 0; then
                echo "+ OK: Files successfully copied.";
                echo ""; 
                echo "";
                RC=0;
            else
                echo "- FAILED: Something went wrong while copying files. Aborting";
                echo "";
                echo "";
                RC=2;
            fi
        #upgrade installation
        else
            echo "+ OK: Upgrade installation";
            ${ACS_AGENT} -f stop
            if test $? -eq 0; then
                echo "+ OK: ACS agent stopped.";             
                copyFiles
                if test $? -eq 0; then
                    echo "+ OK: Files successfully copied.";
                    RC=0;
                    ${ACS_AGENT} -f restart
                    if test $? -eq 0; then
                        echo "+ OK: ACS agent started.";
                        echo "";
                        echo "";
                        RC=0;
                    else
                        echo "- FAILED: Couldn't re-start ACS agent.";
                        echo "";                   
                        echo "";
                        RC=2;       
                    fi             	              
                else
                    echo "- FAILED: Something went wrong while copying files. Aborting";
                    echo "";     
                    echo "";         
                    RC=2;
                fi
            else
                echo "- FAILED: Couldn't stop ACS agent ${ACS_AGENT}. Therefore installation not successful.";
                echo "";  
                echo "";          
                RC=2;
            fi
        fi
	else
        echo "- FAILED: Installation couldn't start because some checks failed. Aborting.";
        echo "";   
        echo ""; 
        RC=2;
	fi
	;;
    
    query)
	if check_perm ${ACS_AGENT} verbose; then 
	    RC=0;
	else
	    RC=2;
	fi
	;;
    setup)
# interactive configuration
	if test ! -u ${ACS_AGENT}; then
	    echo "SUID bit for ${ACS_AGENT} is NOT set. Please run '${SCRIPT_NAME} -a enable' as root.";
	    exit 2;
	fi
# check if the wizard function is invoked by the instance owner
	if test `id -u` -eq 0; then
	    echo "Please perform this action as the database instance owner and not as root.";
	    exit 2;
	fi
	ACSD_BIN=${INST_DIR}/acs/acsd
	if ! test -f ${ACSD_BIN} -a -x ${ACSD_BIN}; then
	    echo "DB2 ACS management daemon (${ACSD_BIN}) cannot be executed. Aborting."
	    exit 2;
	fi
# is customer using full version
	SERVER=PS;
	if test -f ${INST_DIR}/acs/tsm4acs; then
	    if test `uname` = "Linux"; then
# the $ forces the decode of escape chars
		yes_or_no $'Do you want to install TSM for ACS on the production system? [Y/N]\nAnswer "no" if you want to install a backup server.'
	    else
		yes_or_no 'Do you want to install TSM for ACS on the production system? [Y/N]\nAnswer "no" if you want to install a backup server.'
	    fi
	    if test $? -eq 1; then
		SERVER=BS;
		echo "Going to install on backup server";
		touch ${INST_DIR}/acs/.lock
		chmod 600 ${INST_DIR}/acs/.lock
		INST_DIR_OWN_GRP=`ls -dl ${INST_DIR} | awk '{print $3 ":" $4}'`
	    else
		echo "Going to install on production server";
	    fi
	fi
# check for existing default profile 
	PROFILE=${INST_DIR}/acs/profile;
	WIZARD=1;
	if test -f ${PROFILE}; then
# no update of existing profile on backup server - repo may be locked by instance from prod server
	    if test ${SERVER} = "BS"; then
		WIZARD=0;
	    else
		select_profile_action;
	    fi
	else
	    PROFILE=;
	fi

	if test -z "${PROFILE}"; then
# default should be a dir in the home of the instance owner, not the instance dir itself
	    PROFILE_PATH=${HOME}/acs;
	    read_path;
	    echo "selected profile path: ${PROFILE_PATH}";
	    PROFILE=${PROFILE_PATH}/profile;
# check again if there exists a profile at the selected location
	    if test -f ${PROFILE}; then
	        if test ${SERVER} = "BS"; then
		    WIZARD=0;
	        else
		    select_profile_action;
		fi
	    fi
	fi
	REMOVE_LNK="";
	if test -h ${PROFILE}; then
	    REMOVE_LNK=${PROFILE};
	    read_link ${PROFILE}
	    PROFILE=${LINKED_FILE};
	fi
	if ! ${ACS_AGENT} -f isacsrunning > /dev/null 2>&1; then
	    echo "The DB2 ACS management daemon seems to be running.";
	    yes_or_no "Do you want me to stop it to proceed with the configuration? [Y/N]";
	    if test $? -eq 1; then
		echo "Aborting on your request.";
		exit 2;
	    else
		echo "Stopping DB2 ACS management daemon on your request.";
		${ACS_AGENT} -f stop
		if test $? -ne 0; then
		    echo "Failed to stop the DB2 ACS management daemon. Aborting.";
		    echo "";
		    echo "If you are sharing the <Instance_Directory> of your production system via NFS with this";
		    echo "system, you need to stop TSM for ACS on the production system before invoking";
		    echo "'${SCRIPT_NAME}' on this machine.";
		    echo "";
		    echo "In order to continue please execute the following steps:";
		    echo "    1) log on to the production system and call '${SCRIPT_NAME} -a stop -d <Instance_Directory>'";
		    echo "    2) rerun '${SCRIPT_NAME}' on this machine. Choose the option '(r)euse the profile unchanged.'";
		    echo "    3) run '${SCRIPT_NAME} -a start -d <Instance_Directory>' from the production system ";
		    echo "       to resume TSM for ACS";
		    exit 2;
		fi
	    fi
	fi
	echo "Deactivating DB2 ACS management daemon.";
	${ACS_AGENT} -f uninstall
	if test $? -ne 0; then
	    echo "Failed to deactivate the DB2 ACS management daemon. Aborting.";
	    exit 2;
	fi
	if test -h "${REMOVE_LNK}"; then
	    rm $REMOVE_LNK;
	fi
	if test ${WIZARD} -eq 1; then
	    yes_or_no "Is this DB2 instance used for SAP? [Y/N]"
	    if test $? -eq 0; then
		${ACSD_BIN} -f wizard -p ${PROFILE} -m SAP
	    else
		${ACSD_BIN} -f wizard -p ${PROFILE} -m DB2
	    fi
	    RC=$?
	else
	    RC=0;
	fi
	if test ${RC} -eq 0; then
	    if test "${SERVER}" = "PS"; then
		${ACS_AGENT} -f install_${SERVER} -p ${PROFILE}
		RC=$?
	    else
		install_offload_agent;
		RC=$?
		if test "${OFFLOAD_ARG}" = "nooffload"; then
		    echo "Please make sure that the command '${INST_DIR}/acs/tsm4acs' is started by a scheduler or manually."
		fi
	    fi
	fi
	;;
    *)
    	echo "UNHANDLED ACTION!!!"
	;;
esac

exit ${RC}

